@@ -37,10 +37,11 @@
 #'   \code{"bingp"} or \code{"pp"}, i.e. \emph{not} implemented if
 #'   \code{model = "os"}.
 #' @param thresh A numeric scalar.  Extreme value threshold applied to data.
-#'   Only relevant when \code{model = "gp"} or \code{model = "pp"}.  Must
-#'   be supplied when \code{model = "pp"}.  If \code{model = "gp"} and
-#'   \code{thresh} is not supplied then \code{thresh = 0} is used and
-#'   \code{data} should contain threshold excesses.
+#'   Only relevant when \code{model = "gp"}, \code{"pp"} or \code{"bingp"}.
+#'   Must be supplied when \code{model = "pp"} or \code{"bingp"}.
+#'   If \code{model = "gp"} and \code{thresh} is not supplied then
+#'   \code{thresh = 0} is used and \code{data} should contain threshold
+#'   excesses.
 #' @param noy A numeric scalar. The number of blocks of observations,
 #'   excluding any missing values.  A block is often a year.
 #'   Only relevant, and must be supplied, if \code{model = "pp"}.
@@ -230,7 +231,17 @@
 #' fp <- set_prior(prior = "flat", model = "gp", min_xi = -1)
 #' bp <- set_bin_prior(prior = "jeffreys")
 #' bgpg <- rpost_rcpp(n = 1000, model = "bingp", prior = fp, thresh = u,
-#'   data = gom, bin_prior = bp)
+#'                    data = gom, bin_prior = bp)
+#' plot(bgpg, pu_only = TRUE)
+#' plot(bgpg, add_pu = TRUE)
+#'
+#' # Setting the same binomial (Jeffreys) prior by hand
+#' beta_prior_fn <- function(p, ab) {
+#'   return(stats::dbeta(p, shape1 = ab[1], shape2 = ab[2], log = TRUE))
+#' }
+#' jeffreys <- set_bin_prior(beta_prior_fn, ab = c(1 / 2, 1 / 2))
+#' bgpg <- rpost_rcpp(n = 1000, model = "bingp", prior = fp, thresh = u,
+#'                    data = gom, bin_prior = jeffreys)
 #' plot(bgpg, pu_only = TRUE)
 #' plot(bgpg, add_pu = TRUE)
 #'
@@ -312,8 +323,7 @@ rpost_rcpp <- function(n, model = c("gev", "gp", "bingp", "pp", "os"), data,
     check_model <- "gp"
   }
   if (prior_model != check_model) {
-    warning("Are you sure that the prior is compatible with the model?",
-            immediate. = TRUE)
+    stop("The prior is not compatible with the model.")
   }
   # ---------- Create list of additional arguments to the likelihood --------- #
   # Check that any required arguments to the likelihood are present.
@@ -322,6 +332,9 @@ rpost_rcpp <- function(n, model = c("gev", "gp", "bingp", "pp", "os"), data,
     warning("threshold thresh was not supplied so thresh = 0 is used",
             immediate. = TRUE)
   }
+  if (model == "bingp" & is.null(thresh)) {
+    stop("threshold thresh must be supplied when model is bingp")
+  }
   if (model == "pp" & is.null(thresh)) {
     stop("threshold thresh must be supplied when model is pp")
   }
